IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[cms_fixmaxentries]') AND type in (N'P', N'PC'))
drop procedure [dbo].[cms_fixmaxentries]
go
create procedure [dbo].[cms_fixmaxentries] @p_report_enable_sw int=0
as
set nocount on
begin

	/*
		This procedure can be used to reset the contents of DBO.MAX_ENTRIES table if MAX_ENTRIES table is damaged
		
		MAX_TABLE_NUMBER maintained by MAX_ENTRIES fall into three categories:
			1	-	MAX_TABLE_NUMBER is the primary key of the table found in MAX_ENTRIES.TABLE_NAME
			2	-	MAX_TABLE_NUMBER is the 1st column in the table's compound primary key
			3	-	MAX_TABLE_NUMBER is tracked by another column in the table chosen by the application component
			
		Input: 
			-	@p_report_enable_sw	1 = Rows changed in MAX_ENTRIES will be displayed to the caller displaying table, whether inserted or updated and 
										MAX_TABLE_NUMBER before/after value.
									0 = No report (default)
	*/
		
	declare @v_report_results table (table_name nvarchar(128), old_max_id bigint, new_max_id bigint, changedesc nvarchar(40));
	declare @v_table_name nvarchar(128);
	declare @v_column_name nvarchar(128);
	declare @v_pos int;
	declare @v_max_pos int;
	declare @v_max_id bigint;
	declare @v_str nvarchar(3000);
	declare @v_serverindex bigint;
	declare @v_startvalue bigint;
	declare @v_endvalue bigint;
	declare @v_location nvarchar(50);
	declare @v_rows_changed bigint;
	
	declare @v_ErrorNo nvarchar(20);
	declare @P_ErrorMsg nvarchar(1000);
	declare @v_msg nvarchar(2048);

	create table #max_entries_fix 
		(
			id int identity(1,1), 
			table_name nvarchar(128) collate database_default, 
			column_name nvarchar(128) collate database_default, 
			max_id bigint
		);
	
	set @v_rows_changed = 0;
	
	begin try
	
		set @v_location = 'Populate Table';
		
		insert into #max_entries_fix (table_name, column_name)
		select distinct pk.table_name, c.COLUMN_NAME
			from 	INFORMATION_SCHEMA.TABLE_CONSTRAINTS pk 
			inner join INFORMATION_SCHEMA.KEY_COLUMN_USAGE c
				on c.TABLE_NAME = pk.TABLE_NAME and	c.CONSTRAINT_NAME = pk.CONSTRAINT_NAME
			inner join scope_table_map_locker sc
				on pk.table_name = sc.table_name and sc.scope_name in ('ektron','history','notification')
			where pk.CONSTRAINT_TYPE = 'PRIMARY KEY' and 
				c.ordinal_position = 1 and 
				pk.table_name not in(
									'dynamic_data_tbl',				-- Case 3
									'LocalizationObjectTable',		-- Case 3
									'settings',						-- SETTINGS will never have more than one row and doesn't apply to MAX_ENTRIES processing
									'max_entries',					-- MAX_ENTRIES itself doesn't apply to MAX_ENTRIES processing
									'cmsentity_syncstatus_tbl',		-- Doesn't apply to MAX_ENTRIES processing
									'cms_entitytype_tbl',			-- Doesn't apply to MAX_ENTRIES processing
									'CmsPackage_tbl',				-- Doesn't apply to MAX_ENTRIES processing
									'UrlAliasRegEx_tbl',			-- Case 3
									'ad_mapping_tbl',				-- Doesn't apply to MAX_ENTRIES processing
									'AssetDataTable',				-- Doesn't apply to MAX_ENTRIES processing
									'AssetRevisionTable',			-- Doesn't apply to MAX_ENTRIES processing	
									'UrlAliasSettings_tbl',
									'DefaultTrackingProfile',
									'search_mappings',
									'history_meta_tbl',
									'search_suggestedresult', 
									'sitealias_tbl'					-- Doesn't apply to MAX_ENTRIES processing
									)
			order by 1;
			
		insert into #max_entries_fix (table_name, column_name)
		select 'urlaliasmappingManual', 'urlaliasmappingid' 
		 where not exists (select 'x' from #max_entries_fix where table_name = 'urlaliasmappingManual');
		 

		/* Determine valid range of id values for MAX_ENTRIES.MAX_TABLE_NUMBER based on the multisite SETTINGS.SERVER_INDEX value */
		
		set @v_location = 'Get Server Index Range';
		
		select @v_serverindex = coalesce(server_index,1) from settings;
		set @v_startvalue=(@v_serverindex-1)*2147483647+1
		set @v_endvalue=(@v_serverindex)*2147483647

		select @v_pos = min(id), @v_max_pos = max(id) from #max_entries_fix;

		/*
			For Cases 1 & 2 determine the proper value for each row of MAX_ENTRIES by evaluating the current maximum key value
			within the range specified by the SERVER_INDEX for this site.
		*/
		set @v_location = 'New MAX_TABLE_NUMBER for CASE 1 & 2';
		
		while (@v_pos <= @v_max_pos)
		begin
			
			select @v_table_name = table_name, @v_column_name = column_name from #max_entries_fix where id = @v_pos;
			
			if (@v_table_name = 'users') 
				set @v_str='select @v_max_id = max('+@v_column_name+') from '+@v_table_name+' where '+@v_column_name+' <>18611864 and '+@v_column_name+'<>999999999 and '+@v_column_name+' between '+convert(nvarchar,@v_startvalue)+' and '+convert(nvarchar,@v_endvalue)
			else if (@v_table_name='User_Default_Preference_tbl') 
				set @v_str='select @v_max_id = max('+@v_column_name+') from '+@v_table_name+' where '+@v_column_name+' <>18611864 and '+@v_column_name+' <>999999999 and '+@v_column_name+' between '+convert(nvarchar,@v_startvalue)+' and '+convert(nvarchar,@v_endvalue)
			else if (@v_table_name='user_to_group_tbl') 
				set @v_str='select @v_max_id = max('+@v_column_name+') from '+@v_table_name+' where '+@v_column_name+' <>18611864 and '+@v_column_name+' <>999999999 and '+@v_column_name+' between '+convert(nvarchar,@v_startvalue)+' and '+convert(nvarchar,@v_endvalue)
			else if (@v_table_name='usergroups')
				set @v_str='select @v_max_id = max('+@v_column_name+') from '+@v_table_name+' where usergroup_id<>888888 and '+@v_column_name+' between '+convert(nvarchar,@v_startvalue)+' and '+convert(nvarchar,@v_endvalue)
			else if (@v_table_name='urlaliasmapping')
				set @v_str='select @v_max_id = coalesce(max('+@v_column_name+'),'+convert(nvarchar,(@v_endvalue*-1))+') from '+@v_table_name+' where aliastypeid in (1,2,4,5) and '+@v_column_name+' between '+convert(nvarchar,(@v_endvalue*-1))+' and '+convert(nvarchar,(@v_startvalue*-1))
			else if (@v_table_name='urlaliasmappingManual')
				set @v_str='select @v_max_id = max('+@v_column_name+') from urlaliasmapping where aliastypeid = 3 and '+@v_column_name+' between '+convert(nvarchar,@v_startvalue)+' and '+convert(nvarchar,@v_endvalue)
			else
				set @v_str='select @v_max_id = max('+@v_column_name+') from '+@v_table_name+' where '+@v_column_name+' between '+convert(nvarchar,@v_startvalue)+' and '+convert(nvarchar,@v_endvalue)
			--print @v_str;
			
			exec sp_executesql @v_str ,N'@v_max_id bigint out', @v_max_id  out;
		 
			update #max_entries_fix set max_id = coalesce(@v_max_id,@v_startvalue) where id = @v_pos;
			
			set @v_max_id = 0;
			
			set @v_pos = @v_pos + 1;

		end 
		
		/*
			Case 3 tables
		*/
		set @v_location = 'New MAX_TABLE_NUMBER for CASE 3';
		
		insert into #max_entries_fix (table_name, column_name, max_id) 
			select 'dynamic_data_tbl','id',coalesce(max(id),@v_startvalue) from dynamic_data_tbl
				where id between @v_startvalue and @v_endvalue;
				
		insert into #max_entries_fix (table_name, column_name, max_id) 
			select 'LocalizationObjectTable','id',coalesce(max(id),@v_startvalue) from LocalizationObjectTable
				where id between @v_startvalue and @v_endvalue;
				
		insert into #max_entries_fix (table_name, column_name, max_id) 
			select 'UrlAliasRegEx_tbl','RegExId',coalesce(max(RegExId),@v_startvalue) from UrlAliasRegEx_tbl
				where RegExId between @v_startvalue and @v_endvalue;
		
		/*
			Remove extra rows inserted to MAX_ENTRIES (change in 8.5) in preparation for update of MAX_TABLE_NUMBER
		*/
		set @v_location = 'Remove multiple rows in MAX_ENTRIES table';
		
		with last_row_inserted (table_name, max_table_number)
		as
		(
			select table_name, max(max_table_number) from max_entries group by table_name
		)
		delete from max_entries 
			where exists (select 'x' from last_row_inserted lr 
							where lr.table_name = max_entries.table_name and 
								max_entries.max_table_number < lr.max_table_number);
		
		/*
			Replace broken rows with the newly generated ones
		*/
		set @v_location = 'Update MAX_ENTRIES with new value';
		
		update me
			set max_table_number = mef.max_id
		output inserted.table_name, deleted.max_table_number, inserted.max_table_number, 'Updated' into @v_report_results
		from max_entries me
		inner join #max_entries_fix mef
			on mef.table_name = me.table_name and me.max_table_number < mef.max_id ;
			
		update me set max_table_number = mef.max_id 
		output inserted.table_name, deleted.max_table_number, inserted.max_table_number, 'Updated' into @v_report_results
		from max_entries me inner join #max_entries_fix mef on mef.table_name = me.table_name and mef.table_name = 'urlaliasmapping';
			
		set @v_rows_changed = @@rowcount;
		
		/*
			Insert missing rows
		*/
		set @v_location = 'Insert rows to MAX_ENTRIES with new value';
		
		insert into max_entries (table_name, max_table_number)
		 output inserted.table_name, 0, inserted.max_table_number, 'Inserted' into @v_report_results
			select table_name, max_id
			from #max_entries_fix mef
			where not exists (select 'x' from max_entries me where me.table_name = mef.table_name);
		
		set @v_rows_changed = @v_rows_changed + @@rowcount;
		
		/*
			Return resultset with changed rows if @p_report_enable_sw = 1
		*/
		set @v_location = 'Report Changes';
		
		if (@v_rows_changed > 0 and @p_report_enable_sw = 1)
			select changedesc as change_type, table_name as changed_table_row, old_max_id as old_max_number, new_max_id as new_max_number
			from @v_report_results order by 1,2;
		else if (@v_rows_changed = 0 and @p_report_enable_sw = 1)
			select 'no rows updated' as msg;
	
	end try
	begin catch
	
		set @v_ErrorNo = cast(ERROR_NUMBER()  as nvarchar(20));
		set @P_ErrorMsg = cast(ERROR_MESSAGE() as nvarchar(1000));

		set @v_msg = 'Ektron Error in Procedure: cms_fixmaxentries, Table: %s, Error: %s - %s at: %s';
		raiserror(@v_msg,16,1, @v_table_name, @v_ErrorNo, @P_ErrorMsg, @v_location);

	end catch;
	
	drop table #max_entries_fix;
end
go

exec [cms_fixmaxentries] @p_report_enable_sw=1
go
